/**
 * Copyright 2013 Mateusz Kubuszok
 *
 * <p>Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at</p> 
 * 
 * <p>http://www.apache.org/licenses/LICENSE-2.0</p>
 *
 * <p>Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.</p>
 */
package net.jfwu.synchronization;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;
import static net.jfwu.synchronization.ESynchronizationType.READ_WRITE;

import java.lang.annotation.Documented;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * Used to annotate methods that should be synchronized.
 * 
 * <p>
 * Value can be used to define synchronization level (READ_WRITE by default).
 * </p>
 */
@Retention(RUNTIME)
@Documented
@Target(METHOD)
@Inherited
public @interface SynchronizedAccess {
	ESynchronizationType value() default READ_WRITE;
}
