package net.jfwu.synchronization;

/**
 * Copyright 2013 Mateusz Kubuszok
 *
 * <p>Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at</p> 
 * 
 * <p>http://www.apache.org/licenses/LICENSE-2.0</p>
 *
 * <p>Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.</p>
 */
import static java.lang.System.*;
import static java.lang.Thread.sleep;
import static java.util.Collections.synchronizedList;
import static net.jfwu.synchronization.Action.*;
import static net.jfwu.synchronization.ESynchronizationType.*;
import static net.jfwu.synchronization.Moment.*;

import java.util.ArrayList;
import java.util.List;

class SynchronizedClass {
	private final List<History> history = synchronizedList(new ArrayList<History>());
	private String value = "default";

	@SynchronizedAccess(READ)
	public String get() {
		try {
			action(GET, START, value);
			sleepAWhile();
			return value;
		} finally {
			sleepAWhile();
			action(GET, END, value);
		}
	}

	@SynchronizedAccess(WRITE)
	public void set(String value) {
		String old = this.value;
		try {
			action(SET, START, old);
			sleepAWhile();
			this.value = value;
		} finally {
			sleepAWhile();
			action(SET, END, old);
		}
	}

	public List<History> getHistory() {
		return history;
	}

	private void action(Action action, Moment moment, String value) {
		History history = new History(action, moment, value);
		this.history.add(history);
		out.println(currentTimeMillis() + "\t\t" + history.toString());
	}

	private void sleepAWhile() {
		try {
			sleep(300);
		} catch (InterruptedException e) {
		}
	}
}